import { world, system, Player, ItemStack } from "@minecraft/server";

let itemLog = false;
let items = [];
const dimensions = ["overworld", "nether", "the_end"];

const tier15 = [
  "lantern", "ptg:lantern", "beacon", "conduit", "pearlescent_froglight", 
  "verdant_froglight", "ochre_froglight", "glowstone", 
  "lit_pumpkin", "lava_bucket", "campfire", "sea_lantern", "shroomlight"
];
const tier14 = ["torch", "ptg:torch", "glow_berries", "end_rod"];
const tier10 = ["ptg:soul_torch", "soul_torch", "ptg:soul_lantern", "soul_lantern", "soul_campfire", "crying_obsidian"];
const tier7 = ["enchanting_table", "ender_chest", "glow_lichen", "redstone_torch"];
const tier6 = ["sea_pickle", "sculk_catalyst"];
const tier3 = ["magma"];
const tier1 = ["brewing_stand", "dragon_egg", "sculk_sensor"];

function addLightTag() {
  dimensions.forEach(d => {  	
    let dims = world.getDimension(d);
    let items = dims.getEntities({ type: "minecraft:item" });
    items.forEach(itemEntity => {
       if (!itemEntity.isValid()) return;
      const itemComponent = itemEntity.getComponent("item");
      let itemId = itemComponent.itemStack.typeId;
      if (itemId.startsWith("minecraft:")) {
        itemId = itemId.slice(10);
      }
      if (tier15.includes(itemId)) {
        itemEntity.addTag("lv15");
      } else if (tier14.includes(itemId)) {
        itemEntity.addTag("lv14");
      } else if (tier10.includes(itemId)) {
        itemEntity.addTag("lv10");
      } else if (tier7.includes(itemId)) {
        itemEntity.addTag("lv7");
      } else if (tier6.includes(itemId)) {
        itemEntity.addTag("lv6");
      } else if (tier3.includes(itemId)) {
        itemEntity.addTag("lv3");
      } else if (tier1.includes(itemId)) {
        itemEntity.addTag("lv1");
      }
    });
  });
}

function updateEnchantScore(player) {
  if (!player.isValid()) return;
  
  system.run(() => {
    const armourSlots = ["Head", "Chest", "Legs", "Feet", "Mainhand"];
    const excludedItems = ["ptg:torch", "ptg:soul_torch", "ptg:lantern", "ptg:soul_lantern"];
    let enchantScore = 0;

    let equipComponent = player.getComponent("equippable");
    if (!equipComponent) return;

    let mainhandItem = equipComponent.getEquipment("Mainhand");
    let offhandItem = equipComponent.getEquipment("Offhand");

    if (
      (mainhandItem && excludedItems.includes(mainhandItem.typeId)) ||
      (offhandItem && excludedItems.includes(offhandItem.typeId))
    ) {
      enchantScore = 0;
    } else {
      armourSlots.forEach(slot => {
        let armourItem = equipComponent.getEquipment(slot);
        if (armourItem) {
          const enchantComponent = armourItem.getComponent("enchantable");
          if (enchantComponent && enchantComponent.getEnchantments().length > 0) {
            enchantScore += 1;
          }
        }
      });
    }
    let score = world.scoreboard.getObjective("enchant");
    if (score) {
      score.setScore(player, enchantScore);
    }
  });
}

function replaceItems(player) {
if (!player.isValid()) return;
  try {
  const replaceableItems = ["minecraft:torch", "minecraft:soul_torch", "minecraft:lantern", "minecraft:soul_lantern"];
  const customItems = {
    "minecraft:torch": "ptg:torch",
    "minecraft:soul_torch": "ptg:soul_torch",
    "minecraft:lantern": "ptg:lantern",
    "minecraft:soul_lantern": "ptg:soul_lantern"
  };
  
  const inventoryComponent = player.getComponent("minecraft:inventory");
  if (!inventoryComponent) return;
  const inventory = inventoryComponent.container;

  for (let i = 0; i < inventory.size; i++) {
    let itemStack = inventory.getItem(i);
    if (itemStack == undefined) return;
    
    if (itemStack && replaceableItems.includes(itemStack.typeId)) {
      let newItemId = customItems[itemStack.typeId];
      if (newItemId) {
        let newItemStack = new ItemStack(newItemId, itemStack.amount);
        inventory.setItem(i, newItemStack);
      }
    }
  }
} catch (error) {
  console.warn("Error:", error);
  }
}

function glowingEntities() {
dimensions.forEach(d => {
let dims = world.getDimension(d);

let entities = dims.getEntities({excludeTypes: ["minecraft:item", "minecraft:xp_orb"]})
entities.forEach(entity => {
if (entity.isValid()) {
try {
if (entity.getComponent("minecraft:onfire")) {
entity.addTag("onFire")
} else {
entity.removeTag("onFire");
}
} catch {}
}
})
})
}
system.runInterval(() => {
  addLightTag();
  glowingEntities();
  
  const players = world.getAllPlayers();
  
  players.forEach(player => {
    updateEnchantScore(player);
  });
}, 1);

system.runInterval(() => {
  const players = world.getAllPlayers();
  players.forEach(player => {
    if (player.isValid() && player.hasComponent("minecraft:inventory")) {
    replaceItems(player);
    };
  });
});